@@ -22,6 +22,11 @@ from trac.db.util import ConnectionWrapper
 
 _like_escape_re = re.compile(r'([/_%])')
 
+try:
+    import MySQLdb
+    has_mysqldb = True
+except ImportError:
+    has_mysqldb = False
 
 class MySQLConnector(Component):
     """MySQL database support for version 4.1 and greater.
@@ -33,7 +38,11 @@ class MySQLConnector(Component):
     implements(IDatabaseConnector)
 
     def get_supported_schemes(self):
-        return [('mysql', 1)]
+        global has_mysqldb
+        if has_mysqldb:
+            return [('mysql', 1)]
+        else:
+            return []
 
     def get_connection(self, path, user=None, password=None, host=None,
                        port=None, params={}):
@@ -58,14 +67,21 @@ class MySQLConnector(Component):
         a max of 255 bytes per column.
         """
         cols = []
-        limit = 500 / len(columns)
+        limit = 333 / len(columns)
         if limit > 255:
             limit = 255
         for c in columns:
             name = '`%s`' % c
             table_col = filter((lambda x: x.name == c), table.columns)
             if len(table_col) == 1 and table_col[0].type.lower() == 'text':
-                name += '(%s)' % limit
+                if name == '`rev`':
+                    name += '(20)'
+                elif name == '`path`':
+                    name += '(255)'
+                elif name == '`change_type`':
+                    name += '(2)'
+                else:
+                    name += '(%s)' % limit
             # For non-text columns, we simply throw away the extra bytes.
             # That could certainly be optimized better, but for now let's KISS.
             cols.append(name)
@@ -105,7 +121,7 @@ class MySQLConnection(ConnectionWrapper):
         Note that the tuple only checks the major, minor, and sub versions;
         the sub-sub version is weird, so we only check for 'final' versions.
         """
-        from MySQLdb import version_info as ver
+        ver = MySQLdb.version_info
         if ver[0] < v[0] or ver[1] < v[1] or ver[2] < v[2]:
             return False
         if ver[3] != 'final':
@@ -122,8 +138,6 @@ class MySQLConnection(ConnectionWrapper):
 
     def __init__(self, path, user=None, password=None, host=None,
                  port=None, params={}):
-        import MySQLdb
-
         if path.startswith('/'):
             path = path[1:]
         if password == None:
@@ -143,6 +157,7 @@ class MySQLConnection(ConnectionWrapper):
                                   host=host, port=port, use_unicode=True)
             self._set_character_set(cnx, 'utf8')
         ConnectionWrapper.__init__(self, cnx)
+        self._is_closed = False
 
     def cast(self, column, type):
         if type == 'int':
@@ -162,5 +177,17 @@ class MySQLConnection(ConnectionWrapper):
         return self.cnx.insert_id()
 
     def rollback(self):
-        self.cnx.rollback()
         self.cnx.ping()
+        self._set_character_set(self.cnx, 'utf8')
+        try:
+            self.cnx.rollback()
+        except MySQLdb.ProgrammingError:
+            self._is_closed = True
+
+    def close(self):
+        if not self._is_closed:
+            try:
+                self.cnx.close()
+            except MySQLdb.ProgrammingError:
+                pass # this error would mean it's already closed.  So, ignore
+            self._is_closed = True
