@@ -283,7 +283,7 @@ class Ticket(object):
                            "SELECT time,author,'comment',null,description,0 "
                            "FROM attachment WHERE id=%s AND time=%s "
                            "ORDER BY time",
-                           (self.id, when, str(self.id), when, self.id, when))
+                           (self.id, when, str(self.id), when, str(self.id), when))
         else:
             cursor.execute("SELECT time,author,field,oldvalue,newvalue,1 "
                            "FROM ticket_change WHERE ticket=%s "
@@ -293,7 +293,7 @@ class Ticket(object):
                            "UNION "
                            "SELECT time,author,'comment',null,description,0 "
                            "FROM attachment WHERE id=%s "
-                           "ORDER BY time", (self.id,  str(self.id), self.id))
+                           "ORDER BY time", (self.id,  str(self.id), str(self.id)))
         log = []
         for t, author, field, oldvalue, newvalue, permanent in cursor:
             log.append((int(t), author, field, oldvalue or '', newvalue or '',
