@@ -32,11 +32,13 @@ except:
 from trac.log import logger_factory
 from trac.test import TestSetup
 from trac.core import TracError
-from trac.versioncontrol import Changeset, Node
+from trac.versioncontrol import Changeset, Node, NoSuchChangeset
 from trac.versioncontrol.svn_fs import SubversionRepository
 
 REPOS_PATH = os.path.join(tempfile.gettempdir(), 'trac-svnrepos')
 
+HEAD = 21
+
 
 class SubversionRepositoryTestSetup(TestSetup):
 
@@ -94,10 +96,10 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual('trunk', self.repos.normalize_path('/trunk/'))
 
     def test_repos_normalize_rev(self):
-        self.assertEqual(19, self.repos.normalize_rev('latest'))
-        self.assertEqual(19, self.repos.normalize_rev('head'))
-        self.assertEqual(19, self.repos.normalize_rev(''))
-        self.assertEqual(19, self.repos.normalize_rev(None))
+        self.assertEqual(HEAD, self.repos.normalize_rev('latest'))
+        self.assertEqual(HEAD, self.repos.normalize_rev('head'))
+        self.assertEqual(HEAD, self.repos.normalize_rev(''))
+        self.assertEqual(HEAD, self.repos.normalize_rev(None))
         self.assertEqual(11, self.repos.normalize_rev('11'))
         self.assertEqual(11, self.repos.normalize_rev(11))
 
@@ -105,21 +107,22 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(1, self.repos.oldest_rev)
         self.assertEqual(None, self.repos.previous_rev(0))
         self.assertEqual(None, self.repos.previous_rev(1))
-        self.assertEqual(19, self.repos.youngest_rev)
+        self.assertEqual(HEAD, self.repos.youngest_rev)
         self.assertEqual(6, self.repos.next_rev(5))
         self.assertEqual(7, self.repos.next_rev(6))
         # ...
-        self.assertEqual(None, self.repos.next_rev(19))
+        self.assertEqual(None, self.repos.next_rev(HEAD))
+        self.assertRaises(NoSuchChangeset, self.repos.normalize_rev, HEAD + 1)
 
     def test_rev_path_navigation(self):
         self.assertEqual(1, self.repos.oldest_rev)
         self.assertEqual(None, self.repos.previous_rev(0, 'trunk'))
         self.assertEqual(None, self.repos.previous_rev(1, 'trunk'))
-        self.assertEqual(19, self.repos.youngest_rev)
+        self.assertEqual(HEAD, self.repos.youngest_rev)
         self.assertEqual(6, self.repos.next_rev(5, 'trunk'))
         self.assertEqual(13, self.repos.next_rev(6, 'trunk'))
         # ...
-        self.assertEqual(None, self.repos.next_rev(19, 'trunk'))
+        self.assertEqual(None, self.repos.next_rev(HEAD, 'trunk'))
         # test accentuated characters
         self.assertEqual(None, self.repos.previous_rev(17, u'trunk/R\xe9sum\xe9.txt'))
         self.assertEqual(17, self.repos.next_rev(16, u'trunk/R\xe9sum\xe9.txt'))
@@ -127,14 +130,15 @@ class SubversionRepositoryTestCase(unittest.TestCase):
     def test_has_node(self):
         self.assertEqual(False, self.repos.has_node('/trunk/dir1', 3))
         self.assertEqual(True, self.repos.has_node('/trunk/dir1', 4))
+        self.assertEqual(True, self.repos.has_node('/trunk/dir1'))
         
     def test_get_node(self):
         node = self.repos.get_node('/trunk')
         self.assertEqual('trunk', node.name)
         self.assertEqual('/trunk', node.path)
         self.assertEqual(Node.DIRECTORY, node.kind)
-        self.assertEqual(19, node.rev)
-        self.assertEqual(1159353687L, node.last_modified)
+        self.assertEqual(HEAD, node.rev)
+        self.assertEqual(1177955126, node.last_modified)
         node = self.repos.get_node('/trunk/README.txt')
         self.assertEqual('README.txt', node.name)
         self.assertEqual('/trunk/README.txt', node.path)
@@ -159,10 +163,11 @@ class SubversionRepositoryTestCase(unittest.TestCase):
     def test_get_dir_entries(self):
         node = self.repos.get_node('/trunk')
         entries = node.get_entries()
-        self.assertEqual('mpp_proc', entries.next().name)
-        self.assertEqual(u'R\xe9sum\xe9.txt', entries.next().name)
         self.assertEqual('dir1', entries.next().name)
+        self.assertEqual('mpp_proc', entries.next().name)
+        self.assertEqual('v2', entries.next().name)
         self.assertEqual('README3.txt', entries.next().name)
+        self.assertEqual(u'R\xe9sum\xe9.txt', entries.next().name)
         self.assertEqual('README.txt', entries.next().name)
         self.assertRaises(StopIteration, entries.next)
 
@@ -438,6 +443,11 @@ class SubversionRepositoryTestCase(unittest.TestCase):
                          changes.next())
         self.assertRaises(StopIteration, changes.next)
 
+    def test_changeset_utf_8(self):
+        chgset = self.repos.get_changeset(20)
+        self.assertEqual(20, chgset.rev)
+        self.assertEqual(u'Chez moi ça marche\n', chgset.message)
+        self.assertEqual(u'Jonas Borgström', chgset.author)
 
 class ScopedSubversionRepositoryTestCase(unittest.TestCase):
 
@@ -458,10 +468,10 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual('dir1', self.repos.normalize_path('/dir1/'))
 
     def test_repos_normalize_rev(self):
-        self.assertEqual(19, self.repos.normalize_rev('latest'))
-        self.assertEqual(19, self.repos.normalize_rev('head'))
-        self.assertEqual(19, self.repos.normalize_rev(''))
-        self.assertEqual(19, self.repos.normalize_rev(None))
+        self.assertEqual(HEAD, self.repos.normalize_rev('latest'))
+        self.assertEqual(HEAD, self.repos.normalize_rev('head'))
+        self.assertEqual(HEAD, self.repos.normalize_rev(''))
+        self.assertEqual(HEAD, self.repos.normalize_rev(None))
         self.assertEqual(5, self.repos.normalize_rev('5'))
         self.assertEqual(5, self.repos.normalize_rev(5))
 
@@ -469,11 +479,11 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(1, self.repos.oldest_rev)
         self.assertEqual(None, self.repos.previous_rev(0))
         self.assertEqual(1, self.repos.previous_rev(2))
-        self.assertEqual(19, self.repos.youngest_rev)
+        self.assertEqual(HEAD, self.repos.youngest_rev)
         self.assertEqual(2, self.repos.next_rev(1))
         self.assertEqual(3, self.repos.next_rev(2))
         # ...
-        self.assertEqual(None, self.repos.next_rev(19))
+        self.assertEqual(None, self.repos.next_rev(HEAD))
 
     def test_has_node(self):
         self.assertEqual(False, self.repos.has_node('/dir1', 3))
@@ -510,10 +520,11 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
     def test_get_dir_entries(self):
         node = self.repos.get_node('/')
         entries = node.get_entries()
-        self.assertEqual('mpp_proc', entries.next().name)
-        self.assertEqual(u'R\xe9sum\xe9.txt', entries.next().name)
         self.assertEqual('dir1', entries.next().name)
+        self.assertEqual('mpp_proc', entries.next().name)
+        self.assertEqual('v2', entries.next().name)
         self.assertEqual('README3.txt', entries.next().name)
+        self.assertEqual(u'R\xe9sum\xe9.txt', entries.next().name)
         self.assertEqual('README.txt', entries.next().name)
         self.assertRaises(StopIteration, entries.next)
 
@@ -655,6 +666,16 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
                          changes.next())
         self.assertRaises(StopIteration, changes.next)
 
+    def test_changeset_copy_from_outside_and_delete(self):
+        chgset = self.repos.get_changeset(21)
+        self.assertEqual(21, chgset.rev)
+        self.assertEqual('copy from outside of the scope + delete',
+                         chgset.message)
+        changes = chgset.get_changes()
+        self.assertEqual(('v2', 'dir', Changeset.ADD, None, -1),
+                         changes.next())
+        self.assertRaises(StopIteration, changes.next)
+
 
 class RecentPathScopedRepositoryTestCase(unittest.TestCase):
 
