@@ -374,7 +374,7 @@ class Formatter(object):
         else:
             return self._make_intertrac_link(ns, target, label) or \
                    self._make_interwiki_link(ns, target, label) or \
-                   match
+                   escape(match)
 
     def _make_intertrac_link(self, ns, target, label):
         intertrac_config = self.env.config['intertrac']
@@ -721,9 +721,9 @@ class Formatter(object):
             self.in_code_block += 1
             if self.in_code_block == 1:
                 self.code_processor = None
-                self.code_text = ''
+                self.code_text = []
             else:
-                self.code_text += line + os.linesep
+                self.code_text.append(line)
                 if not self.code_processor:
                     self.code_processor = WikiProcessor(self.env, 'default')
         elif line.strip() == Formatter.ENDBLOCK:
@@ -731,20 +731,23 @@ class Formatter(object):
             if self.in_code_block == 0 and self.code_processor:
                 self.close_table()
                 self.close_paragraph()
+                if self.code_text:
+                    self.code_text.append('')
                 self.out.write(to_unicode(self.code_processor.process(
-                    self.req, self.code_text)))
+                    self.req, os.linesep.join(self.code_text))))
+                self.code_text = None
             else:
-                self.code_text += line + os.linesep
+                self.code_text.append(line)
         elif not self.code_processor:
             match = Formatter._processor_re.search(line)
             if match:
                 name = match.group(1)
                 self.code_processor = WikiProcessor(self.env, name)
             else:
-                self.code_text += line + os.linesep 
+                self.code_text.append(line) 
                 self.code_processor = WikiProcessor(self.env, 'default')
         else:
-            self.code_text += line + os.linesep
+            self.code_text.append(line)
 
     def close_code_blocks(self):
         while self.in_code_block > 0:
