@@ -170,8 +170,6 @@ class WikiModule(Component):
                 yield 'wiki', href.wiki(name), title, t, author, comment
 
             # Attachments
-            def display(id):
-                return Markup('ticket ', html.EM('#', id))
             att = AttachmentModule(self.env)
             for event in att.get_timeline_events(req, db, 'wiki', format,
                                                  start, stop,
@@ -257,10 +255,9 @@ class WikiModule(Component):
         if version is not None:
             num_versions = 0
             for v,t,author,comment,ipnr in page.get_history():
-                if v >= old_version:
-                    num_versions += 1;
-                    if num_versions > 1:
-                        break
+                num_versions += 1;
+                if num_versions > 1:
+                    break
             req.hdf['wiki'] = {'version': version, 'old_version': old_version,
                                'only_version': num_versions == 1}
 
